@@ -6,6 +6,10 @@ from meilisearch._httprequests import HttpRequests
 from meilisearch.config import Config
 from meilisearch.task import get_task, get_tasks, wait_for_task
 
+from meilisearch.models.index import IndexStats
+from meilisearch.models.document import Document, DocumentsResults
+from meilisearch.models.task import Task, TaskInfo, TaskResults
+
 # pylint: disable=too-many-public-methods
 class Index():
     """
@@ -71,7 +75,7 @@ class Index():
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
         """
         payload = {'primaryKey': primary_key}
-        return self.http.put(f'{self.config.paths.index}/{self.uid}', payload)
+        return self.http.patch(f'{self.config.paths.index}/{self.uid}', payload)
 
     def fetch_info(self) -> 'Index':
         """Fetch the info of the index.
@@ -124,22 +128,38 @@ class Index():
         payload = {**options, 'uid': uid}
         return HttpRequests(config).post(config.paths.index, payload)
 
-    def get_tasks(self) -> Dict[str, List[Dict[str, Any]]]:
+    def get_tasks(self, parameters: Optional[Dict[str, Any]] = None) -> TaskResults:
         """Get all tasks of a specific index from the last one.
 
+        Parameters
+        ----------
+        parameters (optional):
+            parameters accepted by the get tasks route: https://docs.meilisearch.com/reference/api/tasks.html#get-all-tasks.
+            `indexUid` should be set as a List.
+
         Returns
         -------
-        task:
-            Dictionary containing a list of all enqueued, processing, succeeded or failed tasks of the index.
+        tasks:
+        TaskResults instance with attributes:
+            - from_
+            - next_
+            - limit
+            - results : list of Task instances containing all enqueued, processing, succeeded or failed tasks of the index
 
         Raises
         ------
         MeiliSearchApiError
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
         """
-        return get_tasks(self.config, self.uid)
+        if parameters is not None:
+            parameters.setdefault('indexUid', []).append(self.uid)
+        else:
+            parameters = {'indexUid': [self.uid]}
 
-    def get_task(self, uid: int) -> Dict[str, Any]:
+        tasks = get_tasks(self.config, parameters=parameters)
+        return TaskResults(tasks)
+
+    def get_task(self, uid: int) -> Task:
         """Get one task through the route of a specific index.
 
         Parameters
@@ -150,20 +170,21 @@ class Index():
         Returns
         -------
         task:
-            Dictionary containing information about the processed asynchronous task of an index.
+            Task instance containing information about the processed asynchronous task of an index.
 
         Raises
         ------
         MeiliSearchApiError
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
         """
-        return get_task(self.config, uid, self.uid)
+        task = get_task(self.config, uid)
+        return Task(**task)
 
     def wait_for_task(
         self, uid: int,
         timeout_in_ms: int = 5000,
         interval_in_ms: int = 50,
-    ) -> Dict[str, Any]:
+    ) -> Task:
         """Wait until Meilisearch processes a task until it fails or succeeds.
 
         Parameters
@@ -178,16 +199,17 @@ class Index():
         Returns
         -------
         task:
-            Dictionary containing information about the processed asynchronous task.
+            Task instance containing information about the processed asynchronous task.
 
         Raises
         ------
         MeiliSearchTimeoutError
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
         """
-        return wait_for_task(self.config, uid, timeout_in_ms, interval_in_ms)
+        task = wait_for_task(self.config, uid, timeout_in_ms, interval_in_ms)
+        return Task(**task)
 
-    def get_stats(self) -> Dict[str, Any]:
+    def get_stats(self) -> IndexStats:
         """Get stats of the index.
 
         Get information about the number of documents, field frequencies, ...
@@ -196,16 +218,17 @@ class Index():
         Returns
         -------
         stats:
-            Dictionary containing stats about the given index.
+            IndexStats instance containing information about the given index.
 
         Raises
         ------
         MeiliSearchApiError
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
         """
-        return self.http.get(
+        stats = self.http.get(
             f'{self.config.paths.index}/{self.uid}/{self.config.paths.stat}'
         )
+        return IndexStats(stats)
 
     def search(self, query: str, opt_params: Optional[Dict[str, Any]] = None) -> Dict[str, Any]:
         """Search in the index.
@@ -239,40 +262,20 @@ class Index():
             body=body
         )
 
-    def get_document(self, document_id: str) -> Dict[str, Any]:
+    def get_document(self, document_id: str, parameters: Optional[Dict[str, Any]] = None) -> Document:
         """Get one document with given document identifier.
 
         Parameters
         ----------
         document_id:
             Unique identifier of the document.
-
-        Returns
-        -------
-        document:
-            Dictionary containing the documents information.
-
-        Raises
-        ------
-        MeiliSearchApiError
-            An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
-        """
-        return self.http.get(
-            f'{self.config.paths.index}/{self.uid}/{self.config.paths.document}/{document_id}'
-        )
-
-    def get_documents(self, parameters: Optional[Dict[str, Any]] = None) -> List[Dict[str, Any]]:
-        """Get a set of documents from the index.
-
-        Parameters
-        ----------
         parameters (optional):
-            parameters accepted by the get documents route: https://docs.meilisearch.com/reference/api/documents.html#get-all-documents
+            parameters accepted by the get document route: https://docs.meilisearch.com/reference/api/documents.html#get-one-document
 
         Returns
         -------
         document:
-            List of dictionaries containing the documents information.
+            Document instance containing the documents information.
 
         Raises
         ------
@@ -281,15 +284,51 @@ class Index():
         """
         if parameters is None:
             parameters = {}
-        return self.http.get(
+        elif 'fields' in parameters and isinstance(parameters['fields'], list):
+            parameters['fields'] = ",".join(parameters['fields'])
+
+        document = self.http.get(
+            f'{self.config.paths.index}/{self.uid}/{self.config.paths.document}/{document_id}?{parse.urlencode(parameters)}'
+        )
+        return Document(document)
+
+    def get_documents(self, parameters: Optional[Dict[str, Any]] = None) -> DocumentsResults:
+        """Get a set of documents from the index.
+
+        Parameters
+        ----------
+        parameters (optional):
+            parameters accepted by the get documents route: https://docs.meilisearch.com/reference/api/documents.html#get-documents
+
+        Returns
+        -------
+        documents:
+        DocumentsResults instance with attributes:
+            - total
+            - offset
+            - limit
+            - results : list of Document instances containing the documents information
+
+        Raises
+        ------
+        MeiliSearchApiError
+            An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
+        """
+        if parameters is None:
+            parameters = {}
+        elif 'fields' in parameters and isinstance(parameters['fields'], list):
+            parameters['fields'] = ",".join(parameters['fields'])
+
+        response = self.http.get(
             f'{self.config.paths.index}/{self.uid}/{self.config.paths.document}?{parse.urlencode(parameters)}'
         )
+        return DocumentsResults(response)
 
     def add_documents(
         self,
         documents: List[Dict[str, Any]],
         primary_key: Optional[str] = None,
-    ) -> Dict[str, int]:
+    ) -> TaskInfo:
         """Add documents to the index.
 
         Parameters
@@ -301,8 +340,8 @@ class Index():
 
         Returns
         -------
-        task:
-            Dictionary containing a task to track the informations about the progress of an asynchronous process.
+        task_info:
+            TaskInfo instance containing information about a task to track the progress of an asynchronous process.
             https://docs.meilisearch.com/reference/api/tasks.html#get-one-task
 
         Raises
@@ -311,14 +350,15 @@ class Index():
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
         """
         url = self._build_url(primary_key)
-        return self.http.post(url, documents)
+        add_document_task = self.http.post(url, documents)
+        return TaskInfo(**add_document_task)
 
     def add_documents_in_batches(
         self,
         documents: List[Dict[str, Any]],
         batch_size: int = 1000,
         primary_key: Optional[str] = None,
-    ) -> List[Dict[str, int]]:
+    ) -> List[TaskInfo]:
         """Add documents to the index in batches.
 
         Parameters
@@ -332,8 +372,8 @@ class Index():
 
         Returns
         -------
-        task:
-            List of dictionaries containing a task to track the informations about the progress of an asynchronous process.
+        tasks_info:
+            List of TaskInfo instances containing information about a task to track the progress of an asynchronous process.
             https://docs.meilisearch.com/reference/api/tasks.html#get-one-task
 
         Raises
@@ -343,19 +383,19 @@ class Index():
             Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
         """
 
-        uids = []
+        tasks: List[TaskInfo] = []
 
         for document_batch in self._batch(documents, batch_size):
-            uid = self.add_documents(document_batch, primary_key)
-            uids.append(uid)
+            task = self.add_documents(document_batch, primary_key)
+            tasks.append(task)
 
-        return uids
+        return tasks
 
     def add_documents_json(
         self,
         str_documents: str,
         primary_key: Optional[str] = None,
-    ) -> Dict[str, int]:
+    ) -> TaskInfo:
         """Add string documents from JSON file to the index.
 
         Parameters
@@ -367,8 +407,8 @@ class Index():
 
         Returns
         -------
-        task:
-            Dictionary containing a task to track the informations about the progress of an asynchronous process.
+        task_info:
+            TaskInfo instance containing information about a task to track the progress of an asynchronous process.
             https://docs.meilisearch.com/reference/api/tasks.html#get-one-task
 
         Raises
@@ -382,7 +422,7 @@ class Index():
         self,
         str_documents: str,
         primary_key: Optional[str] = None,
-    ) -> Dict[str, int]:
+    ) -> TaskInfo:
         """Add string documents from a CSV file to the index.
 
         Parameters
@@ -394,8 +434,8 @@ class Index():
 
         Returns
         -------
-        task:
-            Dictionary containing a task to track the informations about the progress of an asynchronous process.
+        task_info:
+            TaskInfo instance containing information about a task to track the progress of an asynchronous process.
             https://docs.meilisearch.com/reference/api/tasks.html#get-one-task
 
         Raises
@@ -409,7 +449,7 @@ class Index():
         self,
         str_documents: str,
         primary_key: Optional[str] = None,
-    ) -> Dict[str, int]:
+    ) -> TaskInfo:
         """Add string documents from a NDJSON file to the index.
 
         Parameters
@@ -421,8 +461,8 @@ class Index():
 
         Returns
         -------
-        task:
-            Dictionary containing a task to track the informations about the progress of an asynchronous process.
+        task_info:
+            TaskInfo instance containing information about a task to track the progress of an asynchronous process.
             https://docs.meilisearch.com/reference/api/tasks.html#get-one-task
 
         Raises
@@ -437,7 +477,7 @@ class Index():
         str_documents: str,
         primary_key: Optional[str] = None,
         content_type: Optional[str] = None,
-    ) -> Dict[str, int]:
+    ) -> TaskInfo:
         """Add string documents to the index.
 
         Parameters
@@ -451,8 +491,8 @@ class Index():
 
         Returns
         -------
-        task:
-            Dictionary containing a task to track the informations about the progress of an asynchronous process.
+        task_info:
+            TaskInfo instance containing information about a task to track the progress of an asynchronous process.
             https://docs.meilisearch.com/reference/api/tasks.html#get-one-task
 
         Raises
@@ -461,13 +501,14 @@ class Index():
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
         """
         url = self._build_url(primary_key)
-        return self.http.post(url, str_documents, content_type)
+        response = self.http.post(url, str_documents, content_type)
+        return TaskInfo(**response)
 
     def update_documents(
         self,
         documents: List[Dict[str, Any]],
         primary_key: Optional[str] = None
-    ) -> Dict[str, int]:
+    ) -> TaskInfo:
         """Update documents in the index.
 
         Parameters
@@ -479,8 +520,8 @@ class Index():
 
         Returns
         -------
-        task:
-            Dictionary containing a task to track the informations about the progress of an asynchronous process.
+        task_info:
+            TaskInfo instance containing information about a task to track the progress of an asynchronous process.
             https://docs.meilisearch.com/reference/api/tasks.html#get-one-task
 
         Raises
@@ -489,14 +530,15 @@ class Index():
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
         """
         url = self._build_url(primary_key)
-        return self.http.put(url, documents)
+        response = self.http.put(url, documents)
+        return TaskInfo(**response)
 
     def update_documents_in_batches(
         self,
         documents: List[Dict[str, Any]],
         batch_size: int = 1000,
         primary_key: Optional[str] = None
-    ) -> List[Dict[str, Any]]:
+    ) -> List[TaskInfo]:
         """Update documents to the index in batches.
 
         Parameters
@@ -510,8 +552,8 @@ class Index():
 
         Returns
         -------
-        task:
-            List of dictionaries containing a task to track the informations about the progress of an asynchronous process.
+        tasks_info:
+            List of TaskInfo instances containing information about a task to track the progress of an asynchronous process.
             https://docs.meilisearch.com/reference/api/tasks.html#get-one-task
 
         Raises
@@ -521,15 +563,15 @@ class Index():
             Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
         """
 
-        uids = []
+        tasks = []
 
         for document_batch in self._batch(documents, batch_size):
-            uid = self.update_documents(document_batch, primary_key)
-            uids.append(uid)
+            update_task = self.update_documents(document_batch, primary_key)
+            tasks.append(update_task)
 
-        return uids
+        return tasks
 
-    def delete_document(self, document_id: str) -> Dict[str, Any]:
+    def delete_document(self, document_id: str) -> TaskInfo:
         """Delete one document from the index.
 
         Parameters
@@ -539,8 +581,8 @@ class Index():
 
         Returns
         -------
-        task:
-            Dictionary containing a task to track the informations about the progress of an asynchronous process.
+        task_info:
+            TaskInfo instance containing information about a task to track the progress of an asynchronous process.
             https://docs.meilisearch.com/reference/api/tasks.html#get-one-task
 
         Raises
@@ -548,11 +590,12 @@ class Index():
         MeiliSearchApiError
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
         """
-        return self.http.delete(
+        response = self.http.delete(
             f'{self.config.paths.index}/{self.uid}/{self.config.paths.document}/{document_id}'
         )
+        return TaskInfo(**response)
 
-    def delete_documents(self, ids: List[str]) -> Dict[str, int]:
+    def delete_documents(self, ids: List[str]) -> TaskInfo:
         """Delete multiple documents from the index.
 
         Parameters
@@ -562,8 +605,8 @@ class Index():
 
         Returns
         -------
-        task:
-            Dictionary containing a task to track the informations about the progress of an asynchronous process.
+        task_info:
+            TaskInfo instance containing information about a task to track the progress of an asynchronous process.
             https://docs.meilisearch.com/reference/api/tasks.html#get-one-task
 
         Raises
@@ -571,18 +614,19 @@ class Index():
         MeiliSearchApiError
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
         """
-        return self.http.post(
+        response = self.http.post(
             f'{self.config.paths.index}/{self.uid}/{self.config.paths.document}/delete-batch',
             ids
         )
+        return TaskInfo(**response)
 
-    def delete_all_documents(self) -> Dict[str, int]:
+    def delete_all_documents(self) -> TaskInfo:
         """Delete all documents from the index.
 
         Returns
         -------
-        task:
-            Dictionary containing a task to track the informations about the progress of an asynchronous process.
+        task_info:
+            TaskInfo instance containing information about a task to track the progress of an asynchronous process.
             https://docs.meilisearch.com/reference/api/tasks.html#get-one-task
 
         Raises
@@ -590,9 +634,10 @@ class Index():
         MeiliSearchApiError
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
         """
-        return self.http.delete(
+        response = self.http.delete(
             f'{self.config.paths.index}/{self.uid}/{self.config.paths.document}'
         )
+        return TaskInfo(**response)
 
     # GENERAL SETTINGS ROUTES
 
@@ -615,7 +660,7 @@ class Index():
             f'{self.config.paths.index}/{self.uid}/{self.config.paths.setting}'
         )
 
-    def update_settings(self, body: Dict[str, Any]) -> Dict[str, int]:
+    def update_settings(self, body: Dict[str, Any]) -> Dict[str, Any]:
         """Update settings of the index.
 
         https://docs.meilisearch.com/reference/api/settings.html#update-settings
@@ -638,12 +683,12 @@ class Index():
         MeiliSearchApiError
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
         """
-        return self.http.post(
+        return self.http.patch(
             f'{self.config.paths.index}/{self.uid}/{self.config.paths.setting}',
             body
         )
 
-    def reset_settings(self) -> Dict[str, int]:
+    def reset_settings(self) -> Dict[str, Any]:
         """Reset settings of the index to default values.
 
         https://docs.meilisearch.com/reference/api/settings.html#reset-settings
@@ -666,8 +711,7 @@ class Index():
     # RANKING RULES SUB-ROUTES
 
     def get_ranking_rules(self) -> List[str]:
-        """
-        Get ranking rules of the index.
+        """Get ranking rules of the index.
 
         Returns
         -------
@@ -683,9 +727,8 @@ class Index():
             self.__settings_url_for(self.config.paths.ranking_rules)
         )
 
-    def update_ranking_rules(self, body: List[str]) -> Dict[str, int]:
-        """
-        Update ranking rules of the index.
+    def update_ranking_rules(self, body: List[str]) -> Dict[str, Any]:
+        """Update ranking rules of the index.
 
         Parameters
         ----------
@@ -703,12 +746,12 @@ class Index():
         MeiliSearchApiError
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
         """
-        return self.http.post(
+        return self.http.put(
             self.__settings_url_for(self.config.paths.ranking_rules),
             body
         )
 
-    def reset_ranking_rules(self) -> Dict[str, int]:
+    def reset_ranking_rules(self) -> Dict[str, Any]:
         """Reset ranking rules of the index to default values.
 
         Returns
@@ -729,8 +772,7 @@ class Index():
     # DISTINCT ATTRIBUTE SUB-ROUTES
 
     def get_distinct_attribute(self) -> Optional[str]:
-        """
-        Get distinct attribute of the index.
+        """Get distinct attribute of the index.
 
         Returns
         -------
@@ -746,9 +788,8 @@ class Index():
             self.__settings_url_for(self.config.paths.distinct_attribute)
         )
 
-    def update_distinct_attribute(self, body: Dict[str, Any]) -> Dict[str, int]:
-        """
-        Update distinct attribute of the index.
+    def update_distinct_attribute(self, body: Dict[str, Any]) -> Dict[str, Any]:
+        """Update distinct attribute of the index.
 
         Parameters
         ----------
@@ -766,12 +807,12 @@ class Index():
         MeiliSearchApiError
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
         """
-        return self.http.post(
+        return self.http.put(
             self.__settings_url_for(self.config.paths.distinct_attribute),
             body
         )
 
-    def reset_distinct_attribute(self) -> Dict[str, int]:
+    def reset_distinct_attribute(self) -> Dict[str, Any]:
         """Reset distinct attribute of the index to default values.
 
         Returns
@@ -792,8 +833,7 @@ class Index():
     # SEARCHABLE ATTRIBUTES SUB-ROUTES
 
     def get_searchable_attributes(self) -> List[str]:
-        """
-        Get searchable attributes of the index.
+        """Get searchable attributes of the index.
 
         Returns
         -------
@@ -809,9 +849,8 @@ class Index():
             self.__settings_url_for(self.config.paths.searchable_attributes)
         )
 
-    def update_searchable_attributes(self, body: List[str]) -> Dict[str, int]:
-        """
-        Update searchable attributes of the index.
+    def update_searchable_attributes(self, body: List[str]) -> Dict[str, Any]:
+        """Update searchable attributes of the index.
 
         Parameters
         ----------
@@ -829,12 +868,12 @@ class Index():
         MeiliSearchApiError
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
         """
-        return self.http.post(
+        return self.http.put(
             self.__settings_url_for(self.config.paths.searchable_attributes),
             body
         )
 
-    def reset_searchable_attributes(self) -> Dict[str, int]:
+    def reset_searchable_attributes(self) -> Dict[str, Any]:
         """Reset searchable attributes of the index to default values.
 
         Returns
@@ -855,8 +894,7 @@ class Index():
     # DISPLAYED ATTRIBUTES SUB-ROUTES
 
     def get_displayed_attributes(self) -> List[str]:
-        """
-        Get displayed attributes of the index.
+        """Get displayed attributes of the index.
 
         Returns
         -------
@@ -872,9 +910,8 @@ class Index():
             self.__settings_url_for(self.config.paths.displayed_attributes)
         )
 
-    def update_displayed_attributes(self, body: List[str]) -> Dict[str, int]:
-        """
-        Update displayed attributes of the index.
+    def update_displayed_attributes(self, body: List[str]) -> Dict[str, Any]:
+        """Update displayed attributes of the index.
 
         Parameters
         ----------
@@ -892,12 +929,12 @@ class Index():
         MeiliSearchApiError
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
         """
-        return self.http.post(
+        return self.http.put(
             self.__settings_url_for(self.config.paths.displayed_attributes),
             body
         )
 
-    def reset_displayed_attributes(self) -> Dict[str, int]:
+    def reset_displayed_attributes(self) -> Dict[str, Any]:
         """Reset displayed attributes of the index to default values.
 
         Returns
@@ -918,8 +955,7 @@ class Index():
     # STOP WORDS SUB-ROUTES
 
     def get_stop_words(self) -> List[str]:
-        """
-        Get stop words of the index.
+        """Get stop words of the index.
 
         Returns
         -------
@@ -935,9 +971,8 @@ class Index():
             self.__settings_url_for(self.config.paths.stop_words)
         )
 
-    def update_stop_words(self, body: List[str]) -> Dict[str, int]:
-        """
-        Update stop words of the index.
+    def update_stop_words(self, body: List[str]) -> Dict[str, Any]:
+        """Update stop words of the index.
 
         Parameters
         ----------
@@ -955,12 +990,12 @@ class Index():
         MeiliSearchApiError
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
         """
-        return self.http.post(
+        return self.http.put(
             self.__settings_url_for(self.config.paths.stop_words),
             body
         )
 
-    def reset_stop_words(self) -> Dict[str, int]:
+    def reset_stop_words(self) -> Dict[str, Any]:
         """Reset stop words of the index to default values.
 
         Returns
@@ -981,8 +1016,7 @@ class Index():
     # SYNONYMS SUB-ROUTES
 
     def get_synonyms(self) -> Dict[str, List[str]]:
-        """
-        Get synonyms of the index.
+        """Get synonyms of the index.
 
         Returns
         -------
@@ -998,9 +1032,8 @@ class Index():
             self.__settings_url_for(self.config.paths.synonyms)
         )
 
-    def update_synonyms(self, body: Dict[str, List[str]]) -> Dict[str, int]:
-        """
-        Update synonyms of the index.
+    def update_synonyms(self, body: Dict[str, List[str]]) -> Dict[str, Any]:
+        """Update synonyms of the index.
 
         Parameters
         ----------
@@ -1018,12 +1051,12 @@ class Index():
         MeiliSearchApiError
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
         """
-        return self.http.post(
+        return self.http.put(
             self.__settings_url_for(self.config.paths.synonyms),
             body
         )
 
-    def reset_synonyms(self) -> Dict[str, int]:
+    def reset_synonyms(self) -> Dict[str, Any]:
         """Reset synonyms of the index to default values.
 
         Returns
@@ -1044,8 +1077,7 @@ class Index():
     # FILTERABLE ATTRIBUTES SUB-ROUTES
 
     def get_filterable_attributes(self) -> List[str]:
-        """
-        Get filterable attributes of the index.
+        """Get filterable attributes of the index.
 
         Returns
         -------
@@ -1061,9 +1093,8 @@ class Index():
             self.__settings_url_for(self.config.paths.filterable_attributes)
         )
 
-    def update_filterable_attributes(self, body: List[str]) -> Dict[str, int]:
-        """
-        Update filterable attributes of the index.
+    def update_filterable_attributes(self, body: List[str]) -> Dict[str, Any]:
+        """Update filterable attributes of the index.
 
         Parameters
         ----------
@@ -1081,12 +1112,12 @@ class Index():
         MeiliSearchApiError
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
         """
-        return self.http.post(
+        return self.http.put(
             self.__settings_url_for(self.config.paths.filterable_attributes),
             body
         )
 
-    def reset_filterable_attributes(self) -> Dict[str, int]:
+    def reset_filterable_attributes(self) -> Dict[str, Any]:
         """Reset filterable attributes of the index to default values.
 
         Returns
@@ -1108,8 +1139,7 @@ class Index():
     # SORTABLE ATTRIBUTES SUB-ROUTES
 
     def get_sortable_attributes(self) -> List[str]:
-        """
-        Get sortable attributes of the index.
+        """Get sortable attributes of the index.
 
         Returns
         -------
@@ -1125,9 +1155,8 @@ class Index():
             self.__settings_url_for(self.config.paths.sortable_attributes)
         )
 
-    def update_sortable_attributes(self, body: List[str]) -> Dict[str, int]:
-        """
-        Update sortable attributes of the index.
+    def update_sortable_attributes(self, body: List[str]) -> Dict[str, Any]:
+        """Update sortable attributes of the index.
 
         Parameters
         ----------
@@ -1145,12 +1174,12 @@ class Index():
         MeiliSearchApiError
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
         """
-        return self.http.post(
+        return self.http.put(
             self.__settings_url_for(self.config.paths.sortable_attributes),
             body
         )
 
-    def reset_sortable_attributes(self) -> Dict[str, int]:
+    def reset_sortable_attributes(self) -> Dict[str, Any]:
         """Reset sortable attributes of the index to default values.
 
         Returns
@@ -1168,6 +1197,183 @@ class Index():
             self.__settings_url_for(self.config.paths.sortable_attributes),
         )
 
+    # TYPO TOLERANCE SUB-ROUTES
+
+    def get_typo_tolerance(self) -> Dict[str, Any]:
+        """Get typo tolerance of the index.
+
+        Returns
+        -------
+        settings: dict
+            Dictionary containing the typo tolerance of the index.
+
+        Raises
+        ------
+        MeiliSearchApiError
+            An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
+        """
+        return self.http.get(
+            self.__settings_url_for(self.config.paths.typo_tolerance)
+        )
+
+    def update_typo_tolerance(self, body: Dict[str, Any]) -> Dict[str, Any]:
+        """Update typo tolerance of the index.
+
+        Parameters
+        ----------
+        body: dict
+            Dictionary containing the typo tolerance.
+
+        Returns
+        -------
+        task:
+            Dictionary containing a task to track the informations about the progress of an asynchronous process.
+            https://docs.meilisearch.com/reference/api/tasks.html#get-one-task
+
+        Raises
+        ------
+        MeiliSearchApiError
+            An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
+        """
+        return self.http.patch(
+            self.__settings_url_for(self.config.paths.typo_tolerance),
+            body
+        )
+
+    def reset_typo_tolerance(self) -> Dict[str, Any]:
+        """Reset typo tolerance of the index to default values.
+
+        Returns
+        -------
+        task:
+            Dictionary containing a task to track the informations about the progress of an asynchronous process.
+            https://docs.meilisearch.com/reference/api/tasks.html#get-one-task
+
+        Raises
+        ------
+        MeiliSearchApiError
+            An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
+        """
+        return self.http.delete(
+            self.__settings_url_for(self.config.paths.typo_tolerance),
+        )
+
+    def get_pagination_settings(self) -> Dict[str, Any]:
+        """Get pagination settngs of the index.
+
+        Returns
+        -------
+        settings: dict
+            Dictionary containing the pagination settings of the index.
+
+        Raises
+        ------
+        MeiliSearchApiError
+            An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
+        """
+        return self.http.get(self.__settings_url_for(self.config.paths.pagination))
+
+    def update_pagination_settings(self, body: Dict[str, Any]) -> Dict[str, Any]:
+        """Update the pagination settings of the index.
+
+        Parameters
+        ----------
+        body: dict
+            Dictionary containing the pagination settings.
+            https://docs.meilisearch.com/reference/api/pagination.html#update-pagination-settings
+
+        Returns
+        -------
+        task:
+            Dictionary containing a task to track the informations about the progress of an asynchronous process.
+            https://docs.meilisearch.com/reference/api/tasks.html#get-one-task
+
+        Raises
+        ------
+        MeiliSearchApiError
+            An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
+        """
+        return self.http.patch(
+            path=self.__settings_url_for(self.config.paths.pagination),
+            body=body
+        )
+
+
+    def reset_pagination_settings(self) -> Dict[str, Any]:
+        """Reset pagination settings of the index to default values.
+
+        Returns
+        -------
+        task:
+            Dictionary containing a task to track the informations about the progress of an asynchronous process.
+            https://docs.meilisearch.com/reference/api/tasks.html#get-one-task
+
+        Raises
+        ------
+        MeiliSearchApiError
+            An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
+        """
+        return self.http.delete(self.__settings_url_for(self.config.paths.pagination))
+
+    def get_faceting_settings(self) -> Dict[str, Any]:
+        """Get the faceting settings of an index.
+
+        Returns
+        -------
+        settings: dict
+            Dictionary containing the faceting settings of the index.
+
+        Raises
+        ------
+        MeiliSearchApiError
+            An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
+        """
+
+        return self.http.get(self.__settings_url_for(self.config.paths.faceting))
+
+    def update_faceting_settings(self, body: Dict[str, Any]) -> Dict[str, Any]:
+        """Update the faceting settings of the index.
+
+        Parameters
+        ----------
+        body: dict
+            Dictionary containing the faceting settings.
+            https://docs.meilisearch.com/reference/api/faceting.html#update-faceting-settings
+
+        Returns
+        -------
+        task:
+            Dictionary containing a task to track the informations about the progress of an asynchronous process.
+            https://docs.meilisearch.com/reference/api/tasks.html#get-one-task
+
+        Raises
+        ------
+        MeiliSearchApiError
+            An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
+        """
+        return self.http.patch(
+            path=self.__settings_url_for(self.config.paths.faceting),
+            body=body
+        )
+
+
+    def reset_faceting_settings(self) -> Dict[str, Any]:
+        """Reset faceting settings of the index to default values.
+
+        Returns
+        -------
+        task:
+            Dictionary containing a task to track the informations about the progress of an asynchronous process.
+            https://docs.meilisearch.com/reference/api/tasks.html#get-one-task
+
+        Raises
+        ------
+        MeiliSearchApiError
+            An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
+        """
+        return self.http.delete(self.__settings_url_for(self.config.paths.faceting))
+
+
     @staticmethod
     def _batch(
         documents: List[Dict[str, Any]], batch_size: int
